@@ -1,5 +1,4 @@
 
-
 /*
  * The Real SoundTracker - Audio handling thread
  *
@@ -51,6 +50,7 @@
 #include "time-buffer.h"
 #include "event-waiter.h"
 #include "gui-settings.h"
+#include "tracer.h"
 
 st_mixer *mixer = NULL;
 st_out_driver *playback_driver = NULL;
@@ -187,7 +187,21 @@ audio_ctlpipe_play_song (int songpos,
 	current_driver_object = playback_driver_object;
 	current_driver = playback_driver;
 	audio_prepare_for_playing();
-	xmplayer_init_play_song(songpos, patpos);
+
+	xmplayer_init_play_song(0, 0, TRUE);
+	
+	if(songpos > 0 || patpos > 0) { /* Tracing!!! */
+	    int i;
+
+	    tracer_setnumch(audio_numchannels);
+	    tracer_trace(playback_driver->get_play_rate(playback_driver_object), songpos, patpos);
+	    xmplayer_init_play_song(songpos, patpos, FALSE);
+	    
+	    for(i = 0; i < audio_numchannels; i++)
+		if(gui_settings.permanent_channels & (1 << i))
+		    mixer->loadchsettings(i);
+	}
+
 	a = AUDIO_BACKPIPE_PLAYING_STARTED;
     } else {
 	a = AUDIO_BACKPIPE_DRIVER_OPEN_FAILED;
@@ -207,7 +221,7 @@ audio_ctlpipe_render_song_to_file (gchar *filename)
     g_assert(xm != NULL);
     g_assert(!playing);
 
-#ifndef NO_AUDIOFILE
+#if USE_SNDFILE || !defined (NO_AUDIOFILE)
     if(file_driver_object != NULL) {
 	driver_out_file.common.destroy(file_driver_object);
     }
@@ -219,7 +233,7 @@ audio_ctlpipe_render_song_to_file (gchar *filename)
 	current_driver = &driver_out_file;
 	audio_prepare_for_playing();
 	playing_noloop = TRUE;
-	xmplayer_init_play_song(0, 0);
+	xmplayer_init_play_song(0, 0, TRUE);
 	a = AUDIO_BACKPIPE_PLAYING_STARTED;
 	audio_restore_priority();
     }
@@ -1053,7 +1067,9 @@ audio_mix (void *dest,
 
 	if(playing_noloop && player_looped) {
 	    // "noloop" mode for file renderer -- make rest of buffer silent
-	    memset(dest, 0, samples_left * ((mixfmt & MIXFMT_16) + 1) + ((mixfmt & MIXFMT_STEREO) + 1));
+	    memset(dest, 0,
+		   samples_left * ((mixfmt & MIXFMT_16) ? 2 : 1)
+		                * ((mixfmt & MIXFMT_STEREO) ? 2 : 1));
 	} else {
 	    dest = mixer_mix(dest, samples_left);
 	}
