@@ -25,6 +25,7 @@
 #ifdef USE_GNOME
 #include <gnome.h>
 #endif
+#include <string.h>
 
 #include "i18n.h"
 #include "audioconfig.h"
@@ -163,7 +164,7 @@ static void
 audioconfig_object_changed (void *a,
 			    void *b)
 {
-    unsigned n = (int)b;
+    unsigned n = GPOINTER_TO_INT(b);
     GList *l;
     gchar *insertbuf[2] = { NULL };
     int i, active = -1;
@@ -239,7 +240,7 @@ audioconfig_initialize_mixer_list (void)
 	    active = i;
 	}
 	insertbuf[0] = (gchar*)mixer->id;
-	insertbuf[1] = (gchar*)mixer->description;
+	insertbuf[1] = gettext((gchar*)mixer->description);
 	gtk_clist_append(GTK_CLIST(audioconfig_mixer_clist), insertbuf);
     }
     gtk_clist_thaw(GTK_CLIST(audioconfig_mixer_clist));
