@@ -2,7 +2,7 @@
 /*
  * The Real SoundTracker - Audio configuration dialog (header)
  *
- * Copyright (C) 1999 Michael Krause
+ * Copyright (C) 1999-2001 Michael Krause
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +27,12 @@
 void        audioconfig_dialog                   (void);
 
 void        audioconfig_load_config              (void);
+void        audioconfig_load_mixer_config        (void);
 void        audioconfig_save_config              (void);
 
 // Currently initialized in main.c
 extern GList *drivers[2];
+extern GList *mixers;
 
 #define DRIVER_OUTPUT 0
 #define DRIVER_INPUT  1
