@@ -39,6 +39,8 @@ enum {
     DIALOG_SAVE_INSTRUMENT,
 
     DIALOG_SAVE_RGN_SAMPLE, // is not included in the "File" tab
+    DIALOG_LOAD_PATTERN,
+    DIALOG_SAVE_PATTERN,
     DIALOG_LAST
 };
 
@@ -55,6 +57,9 @@ gboolean        fileops_page_handle_keys           (int shift,
 void            fileops_open_dialog                (void *dummy,
 						    void *index);
 
-void			fileops_tmpclean 					(void);
+void		fileops_refresh_list 		   (GtkFileSelection *fs,
+						    gboolean grab);
+
+void		fileops_tmpclean                   (void);
 						    
 #endif /* _FILE_OPERATIONS_H */
