@@ -33,9 +33,12 @@ keys_get_key_meaning (guint32 keysym,
 
 #define KEYS_MEANING_TYPE(x) (x >> 16)
 #define KEYS_MEANING_TYPE_MAKE(x) (x << 16)
+#define KEYS_MEANING_VALUE(x) (x & 0xFFFF)
 
 #define KEYS_MEANING_NOTE   0
 #define KEYS_MEANING_KEYOFF 1
+#define KEYS_MEANING_FUNC   2
+#define KEYS_MEANING_CH     3
 
 #define ENCODE_MODIFIERS(shift, ctrl, alt) (1 * (shift != 0) + 2 * (ctrl != 0) + 4 * (alt != 0))
 
