@@ -3,6 +3,7 @@
  * The Real SoundTracker - gtk+ Playlist widget (header)
  *
  * Copyright (C) 1999-2001 Michael Krause
+ * Copyright (C) 2003 Yury Aliaev
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,8 +37,11 @@ struct _Playlist
 {
     GtkVBox widget;
 
-    GtkWidget *spin_songlength, *spin_songpos, *spin_songpat, *spin_restartpos;
-    GtkWidget *ibutton, *dbutton;
+    GtkObject *adj_songpos;
+    GtkWidget *spin_songlength, *spin_songpat, *spin_restartpos;
+    GtkWidget *ibutton, *icbutton, *ifbutton, *dbutton;
+    
+    GtkWidget *numlabels[5], *patlabels[5];
 
     int max_length;
     int min_pattern;
@@ -50,6 +54,7 @@ struct _Playlist
     int current_position;
     int restart_position;
     int signals_disabled;
+    int frozen;
 };
 
 struct _PlaylistClass
@@ -84,4 +89,6 @@ int            playlist_get_position           (Playlist *p);
 void           playlist_set_restart_position   (Playlist *p, int pos);
 int            playlist_get_restart_position   (Playlist *p);
 
+void	       playlist_insert_pattern	       (Playlist *p, int pos, int pat);
+
 #endif /* _PLAYLIST_H */
