@@ -24,7 +24,9 @@
 #ifndef _ST_POLL_H
 #define _ST_POLL_H
 
-#ifdef HAVE_SYS_POLL_H
+#include <config.h>
+
+#ifdef HAVE_POLL
 #include <sys/poll.h>
 #else
 
@@ -45,7 +47,7 @@ struct pollfd
 
 int poll(struct pollfd *ufds, unsigned int nfds, int timeout);
 
-#endif /* HAVE_SYS_POLL_H */
+#endif /* HAVE_POLL */
 
 #if defined(_WIN32)
 
