@@ -328,7 +328,7 @@ sample_display_draw_data (GdkDrawable *win,
 			  int x,
 			  int width)
 {
-    gint16 c, d;
+    gint32 c, d;
     GdkGC *gc;
     const int sh = s->height;
 
@@ -348,25 +348,25 @@ sample_display_draw_data (GdkDrawable *win,
     gc = color ? s->bg_gc : s->fg_gc;
 
     if(s->datatype == 16) {
-	c = -((gint16*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x - 1))];
+	c = ((gint16*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x - 1))];
 	
 	while(width >= 0) {
-	    d = -((gint16*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x))];
+	    d = ((gint16*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x))];
 	    gdk_draw_line(win, gc,
-			  x - 1, ((c + 32768) * sh) >> 16,
-			  x,     ((d + 32768) * sh) >> 16);
+			  x - 1, ((32767 - c) * sh) >> 16,
+			  x,     ((32767 - d) * sh) >> 16);
 	    c = d;
 	    x++;
 	    width--;
 	}
     } else {
-	c = -((gint8*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x - 1))];
+	c = ((gint8*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x - 1))];
 	
 	while(width >= 0) {
-	    d = -((gint8*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x))];
+	    d = ((gint8*)s->data)[OFFSET_RANGE(s->datalen, XPOS_TO_OFFSET(x))];
 	    gdk_draw_line(win, gc,
-			  x - 1, ((c + 128) * sh) >> 8,
-			  x,     ((d + 128) * sh) >> 8);
+			  x - 1, ((127 - c) * sh) >> 8,
+			  x,     ((127 - d) * sh) >> 8);
 	    c = d;
 	    x++;
 	    width--;
