@@ -49,6 +49,9 @@ void          st_set_pattern_length                    (XMPattern *, int);
 void          st_exchange_patterns                     (XM *xm,
 							int p1,
 							int p2);
+gboolean      st_check_if_odd_are_not_empty	       (XMPattern *p);
+void	      st_shrink_pattern			       (XMPattern *p);
+void	      st_expand_pattern			       (XMPattern *p);
 
 /* --- Track functions --- */
 XMNote*       st_dup_track                             (XMNote*, int length);
@@ -62,6 +65,7 @@ gboolean      st_is_empty_track                        (XMNote *notes,
 /* --- Instrument functions --- */
 int           st_instrument_num_samples                (STInstrument *i);
 void          st_clean_instrument                      (STInstrument *i, const char *name);
+void          st_copy_instrument		       (STInstrument *src, STInstrument *dest);
 gboolean      st_instrument_used_in_song               (XM *xm, int instr);
 
 /* --- Sample functions --- */
