@@ -24,6 +24,11 @@
 
 #include <glib.h>
 
+#include "tracker.h"
+
 void        transposition_dialog                   (void);
 
+void        transposition_transpose_selection      (Tracker *t,
+						    int by);
+
 #endif /* _ST_TRANSPOSITION_H */
