@@ -134,6 +134,12 @@ typedef struct XM {
     STInstrument instruments[128];
 } XM;
 
+struct f_n_p {
+    gchar *name;
+    XMPattern *pattern;
+    XM *xm;
+};
+
 #define XM_FLAGS_AMIGA_FREQ               1
 #define XM_FLAGS_IS_MOD                   2
 
@@ -147,6 +153,10 @@ gboolean      xm_load_xi                               (STInstrument *instr,
 							FILE *f);
 gboolean      xm_save_xi                               (STInstrument *instr,
 							FILE *f);
+gboolean      xm_xp_load_header			       (FILE *f, int *length);
+gboolean      xm_xp_load			       (FILE *f, int length,
+							XMPattern *patt, XM *xm);
+void          xm_xp_save			       (gint reply, gpointer data);
 
 static inline int
 xm_get_modified (void)
